package DateTime::Format::Human::Duration::Simple::Locale::en;
use Moose;
use namespace::autoclean;

extends 'DateTime::Format::Human::Duration::Simple::Locale';

__PACKAGE__->meta->make_immutable;

1;
